# Generated by Django 5.1.2 on 2024-11-18 19:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fedj_webfinger", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="alias",
            name="value",
            field=models.CharField(max_length=512),
        ),
        migrations.AlterField(
            model_name="link",
            name="href",
            field=models.CharField(help_text="uri of link", max_length=512),
        ),
        migrations.AlterField(
            model_name="link",
            name="rel",
            field=models.CharField(
                help_text="relationship of link to subject", max_length=512
            ),
        ),
        migrations.AlterField(
            model_name="link",
            name="type",
            field=models.CharField(
                blank=True,
                help_text="The media type of the dereferenced link. http://www.iana.org/assignments/media-types/media-types.xhtml",
                max_length=512,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="linkproperty",
            name="value",
            field=models.CharField(max_length=512),
        ),
        migrations.AlterField(
            model_name="linktitle",
            name="value",
            field=models.CharField(max_length=512),
        ),
        migrations.AlterField(
            model_name="property",
            name="name",
            field=models.CharField(max_length=512, unique=True),
        ),
        migrations.AlterField(
            model_name="property",
            name="value",
            field=models.CharField(max_length=512, unique=True),
        ),
        migrations.AlterField(
            model_name="subject",
            name="value",
            field=models.CharField(max_length=512, unique=True, verbose_name="URI"),
        ),
    ]



"""
Data model for Webfinger

For more details see
https://www.rfc-editor.org/rfc/rfc7033#section-4.4
"""

from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

class Subject(models.Model):

    """https://www.rfc-editor.org/rfc/rfc7033#section-4.4.1
    
   From RFC: The value of the "subject" member is a URI that identifies the entity
   that the JRD describes."""

    value = models.CharField(max_length=512, unique=True, null=False, blank=False, verbose_name="URI")

    def __str__(self):
        return self.value

class Alias(models.Model):
    """https://www.rfc-editor.org/rfc/rfc7033#section-4.4.2
    
    From RFC:  The "aliases" array is an array of zero or more URI strings that
   identify the same entity as the "subject" URI.
    """

    class Meta:
        verbose_name_plural = "Aliases"

    subject = models.ForeignKey(Subject, on_delete=models.CASCADE)
    value = models.CharField(max_length=512, unique=False, null=False, blank=False)


class Property(models.Model):

    """https://www.rfc-editor.org/rfc/rfc7033#section-4.4.3
    
    From RFC: The "properties" object comprises zero or more name/value pairs whose
    names are URIs (referred to as "property identifiers") and whose
    values are strings or null.  Properties are used to convey additional
    information about the subject of the JRD.  As an example, consider
    this use of "properties":

     "properties" : { "http://webfinger.example/ns/name" : "Bob Smith" }
     """

    class Meta:
        verbose_name_plural = "Properties"

    subject = models.ForeignKey(Subject, on_delete=models.CASCADE)
    name = models.CharField(max_length=512, unique=True, null=False, blank=False)
    value = models.CharField(max_length=512, unique=True, null=False, blank=False)

    def save(self, *args, **kwargs):
        """Prevent saving if there's already a LinkTitle
        with this name attached to the link"""
        if Property.objects.filter(subject=self.subject, name=self.name).exists():
            raise ValueError(f"Subject cannot have two Properties with the same name: '{self.name}'")

        super().save(*args, **kwargs)

class Link(models.Model):

    """
    https://www.rfc-editor.org/rfc/rfc7033#section-4.4.4

       The "links" array has any number of member objects, each of which
   represents a link [4].  Each of these link objects can have the
   following members:
    - rel
    - type
    - href
    - titles
    - properties
    """

    subject = models.ForeignKey(
        Subject, on_delete=models.CASCADE, related_name="links"
    )
    rel = models.CharField(max_length=512, null=False, blank=False, help_text="relationship of link to subject")
    href = models.CharField(max_length=512, null=False, blank=False, help_text="uri of link")

    
    type = models.CharField(max_length=512, 
        null=True,
        blank=True,
        help_text="The media type of the dereferenced link. http://www.iana.org/assignments/media-types/media-types.xhtml"
    )

class LinkTitle(models.Model):
    """https://www.rfc-editor.org/rfc/rfc7033#section-4.4.4.4
    
    Internationalized name, value pair, where name is the language
    tag or 'und' and value is a human-readable string that describes the resource
    
    Example from RFC:
    {"en-us" : "The Magical World of Steve"}

    would have name "en-us" and value "The Magical World of Steve"
    """

    link = models.ForeignKey(Link, on_delete=models.CASCADE, related_name="titles")
    name = models.CharField(max_length=32, blank=False, null=False, default="und", help_text="must be a language code")
    value = models.CharField(max_length=512, blank=False, null=False)

    def save(self, *args, **kwargs):
        """Prevent saving if there's already a LinkTitle
        with this name attached to the link"""
        if LinkTitle.objects.filter(link=self.link, name=self.name).exists():
            raise ValueError(f"Link cannot have two LinkTitles with the same name: '{self.name}'")
        
        super().save(*args, **kwargs)


class LinkProperty(models.Model):
    """https://www.rfc-editor.org/rfc/rfc7033#section-4.4.4.5
    """

    link = models.ForeignKey(Link, on_delete=models.CASCADE)
    name = models.CharField(max_length=32, blank=False, null=False)
    value = models.CharField(max_length=512, blank=False, null=False)

    def save(self, *args, **kwargs):
        """Prevent saving if there's already a LinkProperty
        with this name attached to the link"""
        if LinkProperty.objects.filter(link=self.link, name=self.name).exists():
            raise ValueError(f"Link cannot have two LinkProperties with the same name: '{self.name}'")
        
        super().save(*args, **kwargs)



from http import HTTPStatus
from django.http import HttpRequest, HttpResponseForbidden, JsonResponse
from django.views.decorators.http import require_http_methods
from fedj_webfinger.models import Link, LinkProperty, LinkTitle, Subject, Alias

@require_http_methods(["GET"])
def webfinger_index(request: HttpRequest):
    """View for webfinger. Should be at https://{domain}/.well-known/webfinger"""

    # Make sure the client has supplied the required 'resource'
    # query param and return a helpful message if not
    if "resource" not in request.GET:
        return JsonResponse(
            {"error": "missing 'resource' query param"}, status=HTTPStatus.BAD_REQUEST
        )

    # Get the requested resource
    resource = request.GET["resource"]

    # Get the rel filter, for filtering links, if provided,
    # or else None
    rel = request.GET.get("rel", None)

    # Look up the subject and return a 404 if not found
    try:
        subject = Subject.objects.get(value=resource)
    except:
        return JsonResponse(
            {"message": "resource not found"}, status=HTTPStatus.NOT_FOUND
        )
    
    # Get all the Aliases and serialize them for the HTTP
    # response payload
    aliases = Alias.objects.all().filter(subject=subject)
    serialized_aliases = [alias.value for alias in aliases]

    # Get all the Links and serialize them fo the HTTP
    # response payload
    links = Link.objects.all().filter(subject=subject)
    # Conditionally filter links by rel param, if provided
    if rel is not None:
        links = links.filter(rel=rel)

    serialized_links = []

    for link in links:
        serialized_link = {
            "rel": link.rel,
            "href": link.href,
            "type": link.type,
        }

        titles: list[LinkTitle] = LinkTitle.objects.filter(link=link)
        for title in titles:
            if "titles" not in serialized_link:
                serialized_link["titles"] = []
            serialized_link["titles"].append({
                title.name: title.value
            })

        properties: list[LinkProperty] = LinkProperty.objects.filter(link=link)
        for property in properties:
            if "properties" not in serialized_link:
                serialized_link["properties"] = []
            serialized_link["properties"].append({
                property.name: property.value
            })

        serialized_links.append(serialized_link)


    return JsonResponse({
            "subject": subject.value,
            "aliases": serialized_aliases,
            "links": serialized_links
        })



# Generated by Django 5.1.2 on 2024-11-18 21:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "fedj_webfinger",
            "0002_alter_alias_value_alter_link_href_alter_link_rel_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="linkproperty",
            name="name",
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name="linktitle",
            name="link",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="titles",
                to="fedj_webfinger.link",
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-11-18 19:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Link",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("rel", models.TextField(help_text="relationship of link to subject")),
                ("href", models.TextField(help_text="uri of link")),
                (
                    "type",
                    models.TextField(
                        blank=True,
                        help_text="The media type of the dereferenced link. http://www.iana.org/assignments/media-types/media-types.xhtml",
                        null=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Subject",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.CharField(max_length=512, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name="LinkProperty",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        default="und",
                        help_text="must be a language code",
                        max_length=32,
                    ),
                ),
                ("value", models.TextField()),
                (
                    "link",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="fedj_webfinger.link",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="LinkTitle",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        default="und",
                        help_text="must be a language code",
                        max_length=32,
                    ),
                ),
                ("value", models.TextField()),
                (
                    "link",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="fedj_webfinger.link",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Property",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.TextField(unique=True)),
                ("value", models.TextField(unique=True)),
                (
                    "subject",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="fedj_webfinger.subject",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Properties",
            },
        ),
        migrations.AddField(
            model_name="link",
            name="subject",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links",
                to="fedj_webfinger.subject",
            ),
        ),
        migrations.CreateModel(
            name="Alias",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.TextField()),
                (
                    "subject",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="fedj_webfinger.subject",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Aliases",
            },
        ),
    ]



from django.contrib import admin

from .models import LinkProperty, Subject, Alias, Link, Property, LinkTitle

class AliasInline(admin.TabularInline):
    model = Alias
    extra = 1

class PropertyInline(admin.TabularInline):
    model = Property
    extra = 1

class LinkTitleInline(admin.StackedInline):
    model = LinkTitle
    extra = 1

class LinkPropertyInline(admin.StackedInline):
    model = LinkProperty
    extra = 1

class LinkAdmin(admin.ModelAdmin):
    inlines = [LinkTitleInline, LinkPropertyInline]
    model = Link

class LinkInline(admin.StackedInline):
    model = Link
    show_change_link = True
    extra = 1

class SubjectAdmin(admin.ModelAdmin):
    inlines = [AliasInline, PropertyInline, LinkInline]

    pass

admin.site.register(Subject, SubjectAdmin)
admin.site.register(Link, LinkAdmin)
